
/* unit unastvam ; for object magnitude */

/* magnitude of the minor planet
   p minor planet vector from the sun
   r minor planet vector from the earth */

#include <math.h>

#include "unastvam.h"
#include "unforfun.h" /* functions datan2 log10 */
#include "unformus.h" /* const astrunit */

/* magnitude of the minor planet
   p minor planet vector from the sun
   r minor planet vector from the earth */
                                   /* vectors in au */
double togetmau ( double h ,       /* absolute magnitude */
                  double g ,       /* cline parameter */
                  double p[4] ,    /* vector from the Sun */
                  double r[4] )    /* vector from the Earth */
{
  double x,z ;
  double s,t ;
  double a,b ;
  double c,y ;
  double u,w ;
  
  x = sqrt(sqr(p[1])+sqr(p[2])+sqr(p[3])); /* distance from the sun rop */
  z = sqrt(sqr(r[1])+sqr(r[2])+sqr(r[3])); /* distance from the earth ro */
  a = scalarmult(p,r)/(x*z); /* for phase cos */
  b = sqrt(fabs(1.0-a*a)); /* for phase sin */
  s = 0.5*datan2(b,a);     /* half of phase */
  t = sin(s)/cos(s);
  a = 3.33;
  b = 0.63;
  c = pow(t,b);
  u = exp(-a*c);  /* f1 */
  a = 1.87;
  b = 1.22;
  c = pow(t,b);
  if  ( c > 30.0 )
    return (h) ;                         /* simple value */
  w = exp(-a*c);  /* f2 */
  y = u+g*(-u+w);
  return (h+5*log10(z*x)-2.5*log10(y)) ; /* complex value */
}


/* magnitude of the minor planet
   p minor planet vector from the sun
   r minor planet vector from the earth */
                                   /* vectors in km */
double togetvau ( double h ,       /* absolute magnitude */
                  double g ,       /* cline parameter */
                  double rs[4] ,    /* vector from the Sun in km */
                  double re[4] )    /* vector from the observatory in km */
{
  int  i ;       // simple count
  double  p[4] ; // from the Sun in AU
  double  r[4] ; // from the Earth in AU

  for ( i = 1 ; i <= 3 ; i++ )  // all distance to AU
    {
      p[i] = rs[i]/astrunit ; // from km to AU unformus.h refer the Sun
      r[i] = re[i]/astrunit ; // from km to AU unformus.h refer the Earth
    }
  return (togetmau(h,g,p,r)) ; // function here above range in AU
}
