
/* function to find satellites in our square from unitsato.c */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>     /* for open text file */
#include <alloc.h>
#include <process.h>
#include <sys\stat.h>
#include <stdlib.h>
#include <string.h>

#include "unitsato.h"
#include "unforrom.h"  /* void fromfixtoterram for matrix */
#include "unforfun.h"  /* simple functions */
#include "unfornor.h"  /* descart position NORAD elements */
#include "unfortim.h"  /* function transutctotdt */
#include "unforcoo.h"  /* void sphericoorindegree */
#include "unfordat.h"  /* function fromdatetomjd */
#include "unfortoh.h"  /* degree to civil form HH MM SS or +-DD ' " */
#include "unsquare.h"  /* function int booyes in square */

/*
   to obtain
      geocentric J2000 position
      of the observatory in kilometer
   input
      tc     current moment in modified julian day
      xt[4]  geocentric terrestrial observatory position in km
   output
      xg[4]  geocentric J2000 station position kilometer
*/

void  obtogeo ( double  tc ,      // current moment mjd
                double  xt[4] ,   // geoterra position station km
                double  xg[4] )   // output : observatory J2000 km
{
  /* xg[4] geocentric position of the observatory  J2000  km */
  double  ft[4][4] ;  /* matrix to change from fixed to terrestrial system */
  double  tf[4][4] ;  /* matrix to change from terrestrial to fixed system */

  double  tdb ; /* time dynamic barycentric in mjd */

  /* matrix to change from terrestrial to celestial system */

  fromfixtoterram (tc,0.0e0,&tdb,ft) ; /* unforrom.c result matrix ft */
  transpmatr (ft,tf) ; /* unforfun.c matrix from terra to fixed equator */

  /* to change observatory geocentric position to celestial system J2000 */

  multmatrvec (tf,xt,xg) ; /* void unforfun position in km geocentric */
  /* result xg[4] geocentric position of the observatory  J2000  km */
}


/*
 procedure to get from string
 osculating Kepler elements:
 M mean anomaly in degree
 o argument of perihelion in degree
 O ascending node in degree
 i inclination in degree
 e eccentricity
 n mean motion in revolution per day
 dn change pf mean motion
*/

void tlenorad ( char    stnul[1] ,  // input
                char    stone[1] ,  // input
                char    sttwo[1] ,  // input  output line   column  format
                char    snm[1] ,    // number COSPAR    1  10 - 15      6s
                char    sno[1] ,    // number NORAD     1   3 -  8      6s
                char    stn[1] ,    // name             0   1 - 14     14s
                int     *year ,     // year             1  19 - 20      2s
                double  *dayp ,     // day part         1  21 - 32     12s
                double  *elm ,      // element M        2  44 - 51      8s
                double  *elp ,      // element o        2  35 - 42      8s
                double  *elu ,      // element O        2  18 - 25      8s
                double  *eli ,      // element i        2   9 - 16      8s
                double  *ele ,      // element e        2  27 - 33      7s
                double  *eln ,      // element n        2  53 - 63     11s
                double  *den )      // change  n        1  34 - 43     10s
{
  int   i , n ;   // counts length of string
  char  c[80] ;   // current string buffer

  /* name stn may be 14 bytes line 0  1 - 14 */
  for ( i = 1 ; i <= 14 ; i++ )  // let be 14 bytes
    stn[i-1] = ' ';
  n = strlen(stone) ;
  if  ( n > 14 ) n = 14 ;
  for ( i = 1 ; i <= n ; i++ )  // may be 14 bytes
    stn[i-1] = stnul[i-1] ;     // current byte to our string
  stn[14] = '\0' ; // to close string by nullo 14 bytes

  /* snm[6]  number COSPAR    1  10 - 15      6s  */
  for ( i = 10 ; i <=15 ; i++ )  // 6 bytes
    snm[i-10] = stone[i-1] ;     // current byte to our string
  snm[6] = '\0' ; // to close string by nullo

  /* sno[6]  number NORAD     1   3 -  8      6s  */
  for ( i = 3 ; i <= 8 ; i++ )  // 6 bytes
    sno[i-3] = stone[i-1] ;     // current byte to our string
  sno[6] = '\0' ; // to close string by nullo

  // year             1  19 - 20      2s
  for ( i = 19 ; i <= 20 ; i++ )  // 2 bytes
    c[i-19] = stone[i-1] ;
  c[2] = '\0' ; // to close string by nullo
  *year = atoi(c) ;

  // day part         1  21 - 32     12s
  for ( i = 21 ; i <= 32 ; i++ )  // 12 bytes
    c[i-21] = stone[i-1] ;
  c[12] = '\0' ; // to close string by nullo
  *dayp = atof(c) ;

  // change  n        1  34 - 43     10s
  for ( i = 34 ; i <= 43 ; i++ )  // 10 bytes
    c[i-34] = stone[i-1] ;
  c[10] = '\0' ; // to close string by nullo
  *den = atof(c) ;

  // element M        2  44 - 51      8s
  for ( i = 44 ; i <= 51 ; i++ )  // 8 bytes
    c[i-44] = sttwo[i-1] ;
  c[8] = '\0' ; // to close string by nullo
  *elm = atof(c) ;  // mean anomaly in degree

  // element o        2  35 - 42      8s
  for ( i = 35 ; i <= 42 ; i++ )  // 8 bytes
    c[i-35] = sttwo[i-1] ;
  c[8] = '\0' ; // to close string by nullo
  *elp = atof(c) ;  // argument of perihalion in degree

  // element O        2  18 - 25      8s
  for ( i = 18 ; i <= 25 ; i++ )  // 8 bytes
    c[i-18] = sttwo[i-1] ;
  c[8] = '\0' ; // to close string by nullo
  *elu = atof(c) ;  // the ascending node in degree

  // element i        2   9 - 16      8s
  for ( i = 9 ; i <= 16 ; i++ )  // 8 bytes
    c[i-9] = sttwo[i-1] ;
  c[8] = '\0' ; // to close string by nullo
  *eli = atof(c) ;  // inclination in degree

  // element e        2  27 - 33      7s
  c[0] = '0' ;
  c[1] = '.' ;
  for ( i = 27 ; i <= 33 ; i++ )  //  2+7 bytes
    c[i-25] = sttwo[i-1] ;
  c[9] = '\0' ; // to close string by nullo
  *ele = atof(c) ;  // eccentricity

  // element n        2  53 - 63     11s
  for ( i = 53 ; i <= 63 ; i++ )  // 11 bytes
    c[i-53] = sttwo[i-1] ;
  c[11] = '\0' ; // to close string by nullo
  *eln = atof(c) ;  // mean motion in revolution per day

}

/* satellite J2000 position in kilometer
   refer to the observatory on the surface of the Earth  */
                                    // input :
void  satosta ( double  elepo ,     // elements epoch
                double  elm ,       // M     degree
                double  elp ,       // omega degree
                double  elu ,       // Omega degree
                double  eli ,       // i degree
                double  ele ,       // e
                double  eln ,       // n rev./day
                double  den ,       // change n
                double  tfind ,     // moment in modified julian day
                double  xh[4] ,     // geocentric observatory J2000 km
                double  posp[4] )   // output : position satellite
{                 
                          // for the satellite  km km/s
  double  xx[4],vv[4] ;   // geocentric position velosity
  int     i ;             // simple count 
  double  tf[4][4] ;      // matrix to change from true to fixed 

  /* satellite NORAD two-line elements
     with epoch  elepo  to our moment tfind
     to descart  position velosity in km km/s */

  fromnoradtodescart(elepo,elp,elu,eli,ele,elm,eln,den,
                     tfind,xx,vv); /* unfornor */

  /* matrix to change from true to celestial system */

  fromtruetofixequm (tfind,tf) ; /* unforrom.c result matrix tf */

  /* to change satellite true equator position to celestial system J2000 */

  multmatrvec (tf,xx,vv) ; /* void unforfun position in km geocentric */

  for ( i = 1 ; i <= 3 ; i++ )  // for three coordinates in km
    posp[i] = vv[i]-xh[i] ;     // satellite to observatory result

}

/* general procedure to calculate position of the satellites in square */

void  satinsquare ( double  xpos ,   /* observatory terra x in km */
                    double  ypos ,   /* observatory terra y in km */
                    double  zpos ,   /* observatory terra z in km */
                    double  tfind ,  /* moment in mjd to calculate */
                    double  alphc ,  /* center R.A. in degree */
                    double  deltc ,  /* center declination in degree */
                    double  halfs )  /* half of square in degree */
{
  double  xt[4] ;  /* geocentric terrestrial observatoty position in km */
  double  xg[4] ;  /* geocentric position of the station in J2000  km */

  int     txtelm ;     /* text file with nORAD elements for example */

  char    stemp[1] ;   /* simple string to read */
  char    stnul[78] ;  /* length of the nullo  string in the text file */
  char    stone[78] ;  /* length of the first  string in the text file */
  char    sttwo[78] ;  /* length of the second string in the text file */
  int     n ;          /* count for bytes in strings */

  long    ic , nc ;  // count to read text file
                     // variables to read NORAD tle file
  char    snm[6] ;   // number COSPAR
  char    sno[6] ;    // number NORAD tle
  char    stn[14] ;  // name
  int     year ;     // year           
  double  dayp ;     // day with part
  double  elm ;      // element M      
  double  elp ;      // element o      
  double  elu ;      // element O      
  double  eli ;      // element i      
  double  ele ;      // element e      
  double  eln ;      // element n      
  double  den ;      // change  n

  double  elepo ;             /* epoch of the elements in mjd */

  double  posp[4] ; // satellite refer to observatory J2000 in km
                                  // J2000 system  in degree km
  double  alpha , delta , range ; // spheric refer to observatory

  double  ramot , dcmot ; // motion for r.a. decl. in arcsec per hour

  int     khour , kminu ; // civil form for R.A. HH MM of time
  int     ldegr , lminu ; // civil form for Decl DD mm of arc
  double  rasec , desec ; // civil form second of time of arc
  char    chars ;         // signum for declination '+' or '-'

  /* variables to obtain
     motion for r.a. and decl. in arcsec per hour */

  double  dstep = 1.0/1440.0 ;  // one minute in day
  double  tu,ua,ud ;            // moment r.a. decl. one minute before
  double  tz,za,zd ;            // moment r.a. decl. one minute after
  double  xu[4],xz[4] ;         // geocentric observatory position J2000


  xt[1] = xpos ; /* observatory terrestrial x position to array in km */
  xt[2] = ypos ; /* observatory terrestrial y position to array in km */
  xt[3] = zpos ; /* observatory terrestrial z position to array in km */

                          // observatory refer to the geocenter
  obtogeo (tfind,xt,xg) ; // here above position in km J2000 system

  /* a try to open text file with elements */

  txtelm = open("080520ls.tle",
                O_RDONLY|O_TEXT , S_IREAD) ; /* fcntl.h io.h */

  /* to read information for all satellites from this text file */

  ic = 0 ; /* initial nullo count */
  nc = 0 ; /* initial nullo count */

  /* simple title to print results */

  printf("\n object designation                r.a.         decl.");
  printf("        motion ''/s");
  printf("\n name           cospar norad     h  m  s       o  ' ''      ");
  printf("r.a.    decl.\n");
  printf("\n");

  do         /* to read record by record from file to string */
  {
    nc = nc + 1 ; /* the next string */

    /* a try to read three strings
         stnul stone sttwo from text tle file */

    n = 0 ;  // initial nullo bytes
    do       // for the nullo string
      {
        read (txtelm,stemp,1) ; // one byte to temporary string
        stnul[n] = stemp[0] ;   // this byte to fill our string
        n = n+1 ;               // for the next byte place
      }
    while  ( stemp[0] != '\n' ) ;  // the end of current string
    stnul[n-1] = '\0' ; // the end of string

    n = 0 ;  // initial nullo bytes
    do       // for the first string
      {
        read (txtelm,stemp,1) ; // one byte to temporary string
        stone[n] = stemp[0] ;   // this byte to fill our string
        n = n+1 ;               // for the next byte place
      }
    while  ( stemp[0] != '\n' ) ;  // the end of current string
    stone[n-1] = '\0' ; // the end of string

    n = 0 ;  // initial nullo bytes
    do       // for the second string
      {
        read (txtelm,stemp,1) ; // one byte to temporary string
        sttwo[n] = stemp[0] ;   // this byte to fill our string
        n = n+1 ;               // for the next byte place
      }
    while  ( stemp[0] != '\n' ) ;  // the end of current string
    sttwo[n-1] = '\0' ; // the end of string

    /* function above in this unit
       to get value of the elements from string that read */

    tlenorad (stnul,stone,sttwo,     /* input three strings */
                    snm,sno,stn,     /* output number name */
                    &year,&dayp,     /* output date        output */
                    &elm,&elp,&elu,&eli,&ele,&eln,&den) ;  /* elements */

                      /* elements epoch modified julian day */
    elepo = fromdatetomjd ( 1,1,year,0,0,0.0 ) + (dayp-1.0) ; /* unfordat */
                                        /* with part of day */

    /* satellite J2000 position in km
       posp[4] refer to the observatory */

    satosta (elepo,elm,elp,elu,eli,ele,eln,den,
             tfind,xg,posp) ; // void simple here above

                       // satellite position to spheric coordinates
    sphericoorindegree (posp,&alpha,&delta,&range) ; // unforcoo

    /* to check for object to be or not in our square */

    if ( booyes(alpha,delta,0.0,alphc,deltc,halfs,99.0) )  // unsquare.h
      {
        ic = ic + 1 ; /* the next string with our object in square */

        /* result object from observatory in J2000.0 system
           right ascension declination angle values to civil form */

        prhourangle (alpha,&khour,&kminu,&rasec) ; // unfortoh
        prdegrangle (delta,&chars,&ldegr,&lminu,&desec) ; // unfortoh

        /* to obtain motion in arcsec per hour
           for right ascension and declination */

        tu = tfind-dstep ;   // moment one minute before
                             // for the observatory
        obtogeo (tu,xt,xu) ; // geocentric position J2000 kilometer
                             // for the satellite geocentric in km
        satosta (elepo,elm,elp,elu,eli,ele,eln,den,
                 tu,xu,posp) ; // one minute before
        // satellite refer to observatory to spheric coordinates
        sphericoorindegree (posp,&ua,&ud,&range) ; // unforcoo degree

        tz = tfind+dstep ;   // moment one minute after
                             // for the observatory
        obtogeo (tz,xt,xz) ; // geoocentric position J2000 kilometer
                             // for the object geoocentric 
        satosta (elepo,elm,elp,elu,eli,ele,eln,den,
                 tz,xz,posp) ; // one minute after
                       // satellite position to spheric coordinates
        sphericoorindegree (posp,&za,&zd,&range) ; // unforcoo degree
                       // change (motion) for right ascension (r.a.)
        ramot = 3600.0*((za-ua)/(2.0*dstep))/86400.0; // arcsec per second
                       // change (motion) for declination     (decl)
        dcmot = 3600.0*((zd-ud)/(2.0*dstep))/86400.0; // arcsec per second

        printf ("|%14s|%6s|%6s| %2i %2i %5.2f  %1c%2i %2i %4.1f  %7.1f %7.1f\n",
                 stn,snm,sno,           // name COSPAR NORAD
                 khour,kminu,rasec,     // R.A. civil form
                 chars,                 // signum for declination
                 ldegr,lminu,desec,     // Decl civil form
                 ramot,dcmot ) ;        // motion r.a. decl
      }

  }                         // operator do
  while ( !eof (txtelm) ) ; // operator do

  printf ("\n number of objects %8i  from  %8i  in list\n",ic,nc);

  close (txtelm) ;  /* that is all with this text file io.h */

}
