/* unit unsatfur to find satelites in square */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <io.h>

#include "unsatfur.h"

#include "unfordat.h"   /* transmjdtodate procedure */
#include "unforcoo.h"   /* sphericoorindegree function */
#include "unitsato.h"   /* void satinsquare */

/*
 initial conditions to calculate

   xpos ypos zpos descart position in kilometer
                  of the observatory in terrestrial system

   tfind          in modified julian day
                  moment of time for calculating

   alpha          right ascension (J2000) of the center of square in degree

   delta          declination (J2000) of the center of square in degree

   halfs          half of side for field of view degree

   vmago          magnitude for more brighter objects
*/

void fordemsat ()       /* for the initial conditions to calculate */
{
                         /* observatory position */  
  double  xpos , ypos , zpos ; /* terrestrial system in kilometer */

  double  tfind ; /* moment in mjd to find objects in square */
                          /* center of square in degree */
  double  alpha , delta ; /* geocentric equator J2000 degree  */

  double  halfs ; /*  half of side for field of view in degree */

  double  vmago ; /*  magnitude for more brighter objects */

                  /* temporary variables for civil date in UTC */
  int nd,nm,ny ;  /* day month year */
  int kh,km ;     /* hour minute      or for right ascension */
  double sec ;    /* second           or for declination */

  /* to define observatory position in terrestrial system */
  /* Kislovodsk  3395030.300  3128791.100  4388851.100   2  1  1 */

  xpos = 3395030.3e-3 ; /* in kilometer x position terrestrial */
  ypos = 3128791.1e-3 ; /* in kilometer y position terrestrial */
  zpos = 4388851.1e-3 ; /* in kilometer z position terrestrial */

  /* to define date with part of day */

  /* civil date time in UTC scale */

  nd = 20 ; /* day */
  nm = 5 ;  /* month */
  ny = 2008 ; /* year */
  kh = 20 ; /* hour UTC */
  km = 54 ; /* minute of time UTC */
  sec = 0.0 ; /* second of time for date UTC scale */

  /* from civil date to modified julian day with part of day */

  tfind = fromdatetomjd(nd,nm,ny,kh,km,sec) ; /* function unfordat */


  printf(" \n");
  printf("date    %4i%3i%3i%4i%3i%5.1f  UTC",ny,nm,nd,kh,km,sec);
  printf("  %15.6f  julian day\n",tfind);

  /* to define center of field of view right ascension declination */

  kh = 11 ;    /* right ascension hour */
  km = 28;    /* right ascension minute */
  sec = 0.0 ; /* right ascension second */

  alpha = 15.0*(kh+(km+sec/60.0)/60.0) ; /* center R.A. in degree */

  printf(" \n");
  printf("center :    R.A. =%3ih%2im%4.1fs,",kh,km,sec);

  kh = +1 ;   /* declination degree */
  km = +16 ;   /* declination minute of arc */
  sec = +0.0 ; /* declination second of arc */

  delta = (kh+(km/60.0)/60.0) ; /* center Decl. in degree */

  printf("   Decl. =%3io%2i'%4.1f''\n",kh,km,sec);

  /* to define value half of square for field of view in radian*/

  km = 150 ;  /* value of half of side in minute of arc */

  halfs = (km/60.0) ; /* from arcminute to degree half of side */

  printf(" \n");
  printf("side of square  %8.2f arcminute\n",2.0*km);

  /* to limit value for magnitude */

  vmago = 0.0 ; /* stellar magnitude */

  printf(" \n");
  printf("for objects brighter then %5.1f  stellar magnitude\n",vmago);

  /* function to find satellites in our square from unitsato.c */

  satinsquare (xpos,ypos,zpos,tfind,alpha,delta,halfs) ; /* unitsato */

}
