
/* function to find objects in our square is called from unitasto.c */

#include <math.h>

#include "unsquare.h"
#include "unforfun.h"  // function double datan2 atandegree

/* to change sperical coordinates
      alpha , delta  of the star to ideal ( standart )
      with the centre in point  alphac , deltac :
   r = ( x , y , z ) ,  r1 = ( x1 , y1 , z1 )
   r1 = rx ( pi/2 - deltac ) * rz ( alphac + pi/2 ) * r
   ksi = x1 / z1 , eta = y1 / z1 ,
   algorihm was presented by doctor konstantin v. kuimov */

void fromspherictoidealcoordinates
          ( double  alpha ,    // current R.A. in degree
            double  delta ,    // current Decl in degree
            double  alphac ,   // center  R.A. in degree
            double  deltac ,   // center  Decl in degree
            double  *ksi ,     // ideal coordinates in degree 
            double  *eta )     // ideal coordinates in degree
{
  double  a,d,a0,d0 ;
  double  sa,ca,sd,cd ;
  double  sa0,ca0,sd0,cd0 ;
  double  x,y,z,x1,y1,z1 ;

  double  grarad = M_PI/180.0 ; // from degree to radian

  /* centre position alphac deltac must be known */

  a = grarad*alpha;   // current R.A. to radian
  d = grarad*delta;   // current Decl to radian
  a0 = grarad*alphac; // center R.A. radian
  d0 = grarad*deltac; // center Decl radian

  /* any sinus and cosinus */

  sa = sin(a);
  ca = cos(a);
  sd = sin(d);
  cd = cos(d);
  sa0 = sin(a0);
  ca0 = cos(a0);
  sd0 = sin(d0);
  cd0 = cos(d0);

  /* current rectangular coordinates */

  x = ca*cd;
  y = sa*cd;
  z = sd;

  /* rectangular coordinates around center */

  x1 = -sa0*x+ca0*y;
  y1 = -ca0*sd0*x-sa0*sd0*y+cd0*z;
  z1 = +ca0*cd0*x+sa0*cd0*y+sd0*z;

  /* ideal coordinates result */

  *ksi = (x1/z1)/grarad; // in degree
  *eta = (y1/z1)/grarad; // in degree
}

/* to change ideal coordinates of the star  ksi , eta
      with the center in point  alphac , deltac
      to spheric coordinates  alpha , delta :
   r1 = ( ksi , eta , 1 ) ; r = ( x , y , z )
   r = rz ( - ( pi/2 + alphac ) ) * rx ( - ( pi/2 - deltac ) ) ,
   algorihm was presented by doctor konstantin v. kuimov */

void fromidealtosphericcoordinates
           ( double  ksi ,     // ideal coordinates
             double  eta ,     // in degree
             double  alphac ,  // center
             double  deltac ,  // in degree
             double  *alpha ,  // spheric coordinates
             double  *delta )  // in degree
{
  double  a0,d0,q ;
  double  sa0,ca0,sd0,cd0 ;
  double  x,y,z,x1,y1,z1 ;

  double  grarad = M_PI/180.0 ; // from degree to radian

  /* centre position alphac deltac must be known */
  
  x1 = grarad*ksi;    // ideal to radian
  y1 = grarad*eta;    // rectangular
  z1 = 1.0;           // coordinates
  a0 = grarad*alphac; // center to radian
  d0 = grarad*deltac; // alphac deltac 

  /* any trigonometric actions */

  sa0 = sin(a0);
  ca0 = cos(a0);
  sd0 = sin(d0);
  cd0 = cos(d0);

  /* to rectangular coordinates around center */

  x = -sa0*x1-ca0*sd0*y1+ca0*cd0*z1;
  y = +ca0*x1-sa0*sd0*y1+sa0*cd0*z1;
  z = +cd0*y1+sd0*z1;

  q = sqrt(x*x+y*y); // to spheric coordinates

  *alpha = atandegree(y,x);    // in degree unforfun R.A. result
  *delta = datan2(z,q)/grarad; // in degree unforfun decl result
}

/* if our object is in our square then our function equal 1 */

int  booyes ( double  alpha ,  // current R.A. degree
              double  delta ,  // current Decl degree
              double  vacum ,  // current stellar magnitude
              double  alphc ,  // centre of square R.A. degree
              double  deltc ,  // centre of square Decl degree
              double  halfs ,  // half of side of square degree
              double  vmago )  // limit to stellar magnirude
{
  int     boo ;  // temporary value default false 0

  double  ksi , eta ; // ideal coordinates around center degree
  double  difc , difd , difa ; // value in degree for square
  double  ra,dc ;     // current r.a. decl. in degree from input 

  boo = 0 ;   // default false integer equal 0 nullo

  ra = alpha ; // current right ascension value input parameter
  dc = delta ; // current declination value input parameter

  /* to compare stellar magnitude */

  if  ( vacum > vmago )  return (boo) ; // too fainter object

  difc = 1.1*halfs ; // value in degree to find object in square

  /*
     declination of center point
        may be greater then half of square
     or may be less then ones ,
     for example
       deltc = +2 degree  (declination center)
       halfs = +1 degree  (half of square)
       then
         interval for delta is
         ( +1 <= delta ) and ( delta <= +3 )
       deltc = -2 degree  (declination center)
       halfs = +1 degree  (half of square)
       then
         interval for delta is
         ( -3 <= delta ) and ( delta <= -1 )
  */

  if  ( fabs(deltc) > 1.5*halfs )
    {
      difd = fabs(dc-deltc) ; // difference for declination in degree

      if  ( difd > difc )  return (boo) ; // no object boo = 0 default

      /* to compare right ascension and declination
         by using ideal coordinates around center */

      fromspherictoidealcoordinates // here above
              ( ra ,       // current R.A. in degree
                dc ,       // current Decl in degree
                alphc ,    // center  R.A. in degree
                deltc ,    // center  Decl in degree
                &ksi ,     // ideal coordinates in degree
                &eta ) ;   // ideal coordinates in degree

      if ( fabs(ksi) > difc ) return (boo) ; // out of interval false

      if ( fabs(eta) > difc ) return (boo) ; // out of interval false

      boo = 1 ; // our object is in square  true for function value

      return (boo) ; // result this object is in square
    }

  /* may be other case
     when center for delta near 0.0 degree
     for example
       deltc = +0 degree  (declination center)
       halfs = +1 degree  (half of square)
       then
         interval for delta is
         ( -1 <= delta ) and ( delta <= +1 )
  */

  /* to compare right ascension and declination
     by using ideal coordinates around center along equator */

  fromspherictoidealcoordinates // here above
          ( ra ,       // current R.A. in degree
            dc ,       // current Decl in degree
            alphc ,    // center  R.A. in degree
            deltc ,    // center  Decl in degree
            &ksi ,     // ideal coordinates in degree
            &eta ) ;   // ideal coordinates in degree

  if ( fabs(ksi) > difc ) return (boo) ; // out of interval

  if ( fabs(eta) > difc ) return (boo) ; // out of interval

  /*
     may be a next case
       r.a. center near 0.0 degree
            for example
                alphc = +2  or  alphc = +358 degree
     this is special case to analyse situation
 */

  difa = fabs(ra-alphc) ; // difference for right ascension in degree
                                                  // difference value
  if  ( difa > 330.0 )  difa = fabs(difa-360.0) ; // to correct 
                                      // to compare with half of side
  if  ( difa > difc )  return (boo) ; // no object in our square false

  boo = 1 ; // our object is in square but true for boo

  return (boo) ; // result function value is true
}
